;------------------------------------------------------
; Project: 	DIGITAL CLOCK WITH DATE AND THERMOMETER with THERMOSTAT
;				WITH PIC16F628 and DS1631....... ENJOY! :-D
; File: digital_clock_date_thermometer_thermostat_16f628_ds1631
; Electric diagram: Bogdan Borca/www.bogdi.ro
; Software: Bogdan Borca/www.bogdi.ro
; Controller: PIC16F628
; Frequency: 4MHz
; Version: 16.0
; Date: 13.JAN.2010
; -----------------------------------------------------



	;-------------------------------------------------
	; THERMOMETER DS1631 OPERATIONS I2C
	; CLOCK : used to go up SCL, wait some time and then go down
	; SDA should not change when SCL is up only on START and STOP routines
	;-------------------------------------------------

	;-------------------------------------------------
	; DS1631 SETUP	
	;-------------------------------------------------

DS1631_Write_Configurations
	port_in_out_config SCL_OUT
	call INIT_DS1631
	init_p t_temp, th
	init_v tr, H'A1'
	call write_to_THERMOSTAT_REGISTER	
	init_p t_temp, tl
	init_v tr, H'A2'
	call write_to_THERMOSTAT_REGISTER			; Write TH to high and low thermostat registers of DS1631
	return

CLOCK 
	bsf SCL			; SCL=1
	time_waste H'2'
	bcf SCL			; SCL=0
	time_waste H'2'
	return

	;-------------------------------------------------
	; ACKNOLEDGE
	;-------------------------------------------------

ACK_Master
							; Make SDA output, ACK
	port_in_out_config SDA_OUT
	bcf SDA					; and also SDA = 0
	time_waste H'1'
	call CLOCK
	port_in_out_config SDA_IN
	return

NACK_Master					; Generate a NACK command to the DS1631
							; Make SDA output, ACK
	port_in_out_config SDA_OUT
	bsf SDA			; and also SDA = 1
	time_waste H'1'			; Wait some time
	call CLOCK				; call CLOCK procedure
	bcf SDA			; and also SDA = 0
	time_waste H'1'			; Wait some time	
	return

ACK  
	port_in_out_config SDA_IN
	time_waste H'1'
	call CLOCK
	port_in_out_config SDA_OUT
	time_waste H'1'			; testing, don't know for sure this is needed
	return

	;-------------------------------------------------
	; SEND WHAT, this initialize I2C_Bus with what
	; and calls I2C_tx_byte
	;-------------------------------------------------

SEND macro what
	init_v I2C_Bus, what
	call I2C_tx_byte
	endm

SENDp macro what
	init_p I2C_Bus, what
	call I2C_tx_byte
	endm

	;-------------------------------------------------
	; START : The master generates a START condition by 
	; pulling SDA from high to low while SCL is high
	;-------------------------------------------------

START 
	bsf SDA			; SDA=1
	time_waste H'1'
	bsf SCL			; SCL=1
	time_waste H'1'
	bcf SDA			; SDA=0
	time_waste H'1'
	bcf SCL			; SCL=0
	time_waste H'1'
	return

	;-------------------------------------------------
	; Initiate a start condition with WRITE/READ command
	;-------------------------------------------------

START_w_90h
	call START				; Initiate start command in WRITE MODE
	SEND H'90'				; Access the DS1631.. it is only one on the bus so A2A1A0 = 000
	call ACK				; H'90' = 1001 0000 last 0 means Write
	return	

START_r_91h
	call START				; Initiate start command in READ MODE
	SEND H'91'				; Access the DS1631.. it is only one on the bus so A2A1A0 = 000
	port_in_out_config SDA_IN
							; H'91' = 1001 0001 last 1 means READ, make SDA input
	time_waste H'1'
	call CLOCK				; call clock routine, SDA stays on input state wher Read Command is issued
	return

	;-------------------------------------------------
	; STOP: The master generates a STOP condition by 
	; transitioning SDA from low to high while SCL is high
	;-------------------------------------------------

STOP
	bsf SCL			; SCL=1
	time_waste H'1'
	bsf SDA			; SDA=1
	time_waste H'1'
	return
	
	;-------------------------------------------------
	; TRANSMIT 1 on the SDA line
	;-------------------------------------------------

I2C_tx_bit_1
	bsf SDA			; SDA=1
	time_waste H'1'
	call CLOCK
	goto continue_tx

	;-------------------------------------------------
	; TRANSMIT 0 on the SDA line
	;-------------------------------------------------

I2C_tx_bit_0
	bcf SDA			; SDA=0
	time_waste H'1'
	call CLOCK
	goto continue_tx

	;-------------------------------------------------
	; TRANSMIT BYTE: The master transmits I2C_Bus on the SDA line 
	;-------------------------------------------------

I2C_tx_byte
	init_v I2C_Bit, H'8'
tx	
	btfss I2C_Bus, 7
		goto I2C_tx_bit_0
		goto I2C_tx_bit_1
continue_tx
	rlf I2C_Bus, 1			; Right shift the bus
	btfss STATUS, C
	bcf I2C_Bus, 0
	decfsz I2C_Bit, 1
	goto tx
	return

	;-------------------------------------------------
	; RECEIVE BYTE: The master receives I2C_DataMSB/LSB on the SDA line 
	;-------------------------------------------------

I2C_Rx_byte	
	init_v I2C_Bit, H'8'
rx
	bsf SCL					; SCL=1
	time_waste H'1'			; wait some time
							; Read data when SCL is UP, SDA must be stable, data is changing when SCL
							; is down so it must be read while SCL is UP (1)

	bcf I2C_Bus, 7			; If the bit is clear (0) then clear Bus Bit	
	btfsc SDA				; Test bit that should come from DS1631
	bsf I2C_Bus, 7			; If the bit is set (1) then set the Bus Bit
rlf__
	rlf I2C_Bus, 1			; Right shift the bus
	btfss STATUS, C			; If carry is 1 
	bcf I2C_Bus, 0			; then clear it

	bcf SCL					; SCL=0
	time_waste H'1'

	decfsz I2C_Bit, 1
	goto rx
	return

	;-------------------------------------------------
	; INIT_DS1631: Function or call to the initialization of DS1631 
	; Operations: Write configuration, write thermostat high and low values 
	; are performed
	;-------------------------------------------------

INIT_DS1631	
	call START_w_90h		; Access DS1631.. it is only one on the bus so A2A1A0 = 000
	SEND H'AC'				; ACCESS Config register with the write intentions
	call ACK				; Receive ACK from the DS1631
	SENDp DS1631_Config		; Send configuration to DS1631 H'03' is 0000 0011 which means 9-bit resolution
	call ACK				; DONE|THF|TLF|NVB|R1 |R0 |POL|1SHOT			with POL and 1SHOT activated
							; R   |RO |R/W|RO |R/W|R/W|R/W|R/W
							; 0	  |0  |0  |0  |0  |0  |1  |1	= H'03'	
							; or another configuration on 12-bit resolution
							; DONE|THF|TLF|NVB|R1 |R0 |POL|1SHOT
							; R   |RO |R/W|RO |R/W|R/W|R/W|R/W
							; 0	  |0  |0  |0  |1  |1  |1  |1	= H'0F'	
							; POL=1 means ThermostatOUT is active high
							; 1SHOT = 1. One-Shot Mode. The Start Convert T command initiates
							; a single temperature conversion and then the device goes into a lowpower
							; standby state.
							; R1 = 0 and R0 = 0 is the setting for resolution to 9-bit
	call STOP
	return

read_from_THERMOSTAT_REGISTER
	call START_w_90h		; START/RESTART
	SENDp tr				; SEND command to access thermostat register AA
	call ACK
	call START_r_91h		; RESTART to READ from thermostat register
	call I2C_Rx_byte		; Get MSB from DS1631
	call NACK_Master		; Give a NACK from the master (PIC) to DS1631
	call STOP
	return

write_to_THERMOSTAT_REGISTER
	call START_w_90h		; START/RESTART to WRITE to TH (THERMOSTAT HIGH VALUE)
	SENDp tr				; (ACCESS TH/TL regsiter) Upper/Lower alarm trip point in twos complement format
	call ACK
	SENDp t_temp			; Send first(most) significant byte (MSB) for (Ex) TH=+40gC 0010 1000 H'28' or TL=35gC 
	call ACK				
	SEND H'00'				; Send last significant byte (LSB) for (Ex) TH=+40gC 0000 0000 or TL=35gC 0000 0000
	call ACK
	call STOP
	return

DS1631_start_conversion
	btfss OPTIONS, 3
	call DS1631_Write_Configurations
	bcf OPTIONS, 3

							; start a conversion, I don't want multiple conversions and not gets for the temperature
	call START_w_90h		; START/RESTART to WRITE/GIVE a START CONVERT COMMAND
	SEND H'51'				; SEND START CONVERT COMMAND, this device is oneShot mode active so
;	SEND H'EE'				; for DS1621 compatibility

	call ACK				; will performe only one conversion and then stop
	call STOP
	goto back_from_start_conversion

DS1631_get_temperature
	call START_w_90h		; START/RESTART
	SEND H'AA'				; SEND command to access temperature register AA
	call ACK
	call START_r_91h		; RESTART to READ from temperature register
	call I2C_Rx_byte		; Get MSB from DS1631 into t register
	call NACK_Master		; Give a NACK from the master (PIC) to DS1631
	call STOP

	init_p t, I2C_Bus
							; I don't need LSB in 9/12 bit resolution so call a NACK instead ACK
							; init_p I2C_DataLSB, I2C_Bus
							; will not display the temperature with period EX: 23.5 but only 23

	goto back_from_get_temperature					
							; for one conversion

							; If t = H'24' in hexadecimal then
							; this means in Decimal number 36
							; When divide this at 10 the
							; divide_Result = 3 and divide_Rest = 6
							; so no conversion from binar/hexa to decimal is required