;------------------------------------------------------
; Project: 	DIGITAL CLOCK WITH DATE AND THERMOMETER with THERMOSTAT
;				WITH PIC16F628 and DS1631....... ENJOY! :-D
; File: digital_clock_date_thermometer_thermostat_16f628_ds1631
; Electric diagram: Bogdan Borca/www.bogdi.ro
; Software: Bogdan Borca/www.bogdi.ro
; Controller: PIC16F628
; Frequency: 4MHz
; Version: 16.0
; Date: 13.JAN.2010
; -----------------------------------------------------

;	ERRORLEVEL -302 

	#define SDA PORTA, 0
	#define SCL PORTB, 3	
	#define SDA_IN PORTA, B'00100001'	; SDA_IN let PORTA[5] as input
	#define SDA_OUT PORTA, B'00100000'	; SDA_OUT let PORTA[5] as input

	#define SCL_IN PORTB, B'00001000'	; SCL_IN let PORTB[6] as input
	#define SCL_OUT PORTB, B'00000000' ; SCL_OUT set PORTB[6] as output

	#define BUTTONS_IN TRISB, B'01001000'	; Buttons located on PORTB[3][6]
	#define BUTTONS_OUT TRISB, B'00000000'

	list p=16F628			; Controller Type
	#include p16F628.inc	; inc file from Microchip+

	__CONFIG _MCLRE_OFF & _WDT_OFF & _CP_OFF & _XT_OSC & _LVP_OFF & _PWRTE_ON & _BODEN_OFF

	CBLOCK 0x20		;start to declare variables at address 0x20
		seconds		;from 0 to 59

		;==============================================================================
		; 4 SEGMENTS DISPLAY
		; 88:88 -> the position of segment notation is: seg1seg2 : seg3seg4
		; at address 0x21 I hold segment one (the one from the right, least semnificativ) (1)
		seg1, seg2, seg3, seg4		
		; ex1 showing time: 12.48 where seg1=1 seg2=2 seg3=4 and seg4=8
		; ex2 showing temp:  5gC where seg1=nothing seg2=5 seg3=g seg4=C
		; ex3 showind date: 12.SE where seg1=1 seg2=2 seg3=S seg4=E

		;==============================================================================
		; TIME DISPLAY
		; Ex: 23:19 -> the notation is: h1h2 : m1m2 (h1=2 h2=3 m1=1 m2=9)
		h, m	

		;==============================================================================
		; DATE DISPLAY
		dd, dm, year, days_per_month, date_type
		;day1, range from 0 to x, day2, range from 0 to x, month, range from 0 to 11
		; Ex showind date: 12.SE where d1=1 d2=2 dm+10=S dm+22=E
		
		;==============================================================================
		; COUNTERS
		; TIME LOOP, WRT__CHG, WRT_WHAT
		TL, WRT__CHG, sec_on_off, hold, WRT__WHAT
		;on the display it is shown: time, date, temperature
		;if WRT__WHAT is 1 then the display will show the time
		;if WRT__WHAT is 2 then the display will show the date
		;if WRT__WHAT is 3 then the display will show the temperature
		;if WRT__WHAT is 4 set mode is active and it is written time set, better see test_WW call
		;sec_on_off is used to switch on/off the middle two points
		;sec_on_off is true one seconds and the next is false and so on

		;==============================================================================
		; TEMPERATURE DISPLAY
		t, tl, th, t_temp, tr
		; Ex showing temperature: 
		;	 5gC where t1=nothing t2=5 t3=g C it is shown allways, can be seg4
		;	23gC where t1=2 t2=3 t3=g and C
		;	-23C where t1=- t2=2 t3=3 and C
		; 	-8gC where t1=- t2=8 t3=g and C
		;tl is the low value for the thermostat
		;th is the high value for the thermostat

		;==============================================================================
		; ONE SECOND VARIABLES HERE from http://www.romanblack.com/one_sec.htm
		bres_hi			; hi byte of our 24bit variable
		bres_mid		; mid byte
		bres_lo			; lo byte
						; (we only need 3 bytes for this system)
		bmid, blo		; used in settings for second calibration
		status_temp		; used for interrupt servicing
		w_temp			; used for interrupt servicing
		;==============================================================================
		divide_Rest, divide_Who, divide_Result, divide_At
		;==============================================================================
		I2C_Bus, I2C_Bit, OPTIONS, SERVICE, DS1631_Config
		;==============================================================================
		; TIMERS
		tt, td, te, sec_no
	
	ENDC


;==============================================================================

init_p macro parameter1, parameter2
	movfw parameter2				; make W = parameter2
	movwf parameter1				; make parameter1 = W
	endm

init_v2 macro parameter1, value1, parameter2, value2
									;macro used to init 2 variables							
	init_v parameter1, value1
	init_v parameter2, value2
	endm

init_v macro parameter, value		;macro used to init a variable							
	movlw value						;give W=value
	movwf parameter					;give parameter=W
	endm

add macro from_parameter, value, to_parameter
	movf from_parameter, 0
	addlw value
	movwf to_parameter	
	endm

time_waste macro how_much
	clrw
	addlw how_much		
	call wasteit		
	endm

_if macro parameter, value, address
	movfw parameter
	sublw value
	btfsc STATUS, Z	
	call address
	endm

port_in_out_config macro TRISAB, config
	bsf STATUS, RP0				; bank 1
	movlw config
	movwf TRISAB
	bcf	STATUS, RP0				; bank 0
	endm

divide macro Who_, At_
	init_p divide_Who, Who_
	init_v divide_At, At_
	call divide_call
	endm

;==============================================================================
; Code here

	org 0x000 			;set program memory base at reset vector 0x000
reset
	clrf TMR0
	goto setup			;set up ints and port stuff

	org 0x004			;interrupt vector, int handler code comes next.
;==============================================================================

;******************************************************************************
; INTERRUPT HANDLER     (runs this code each timer0 interrupt)
; Many thanks to Roman Black for the second event http://www.romanblack.com/one_sec.htm
;******************************************************************************
;
;------------------
int_handler				;
;------------------

	;-------------------------------------------------
						; first we preserve w and status register

	movwf w_temp      	; save off current W register contents
	movf STATUS, w		; move status register into W register
	movwf status_temp	; save off contents of STATUS register

	;-------------------------------------------------
	; Note! we get here every 256 instructions, we
	; can now do our special one second timing system.				

	; This consists of three main steps;
	; * subtract 256 counts from our 24bit variable
	; * test if we reached the setpoint
	; * if so, add 1,000,000 counts to 24bit variable and generate event.
	;-------------------------------------------------
						; * optimised 24 bit subtract here 
						; This is done with the minimum instructions.
						; We subtract 256 from the 24bit variable
						; by just decrementing the mid byte.

	tstf bres_mid		; first test for mid==0
	skpnz				; nz = no underflow needed
	decf bres_hi,f		; z, so is underflow, so dec the msb

	decfsz bres_mid,f	; dec the mid byte (subtract 256)

						; now the full 24bit optimised subtract is done!
						; this is about 4 times faster than a "proper"
						; 24bit subtract.

	goto int_exit			; nz, so definitely not one second yet.
						; in most cases the entire int takes
						; only 16 instructions.
	;-------------------------------------------------
						; * test if we have reached one second.
						; only gets here when mid==0, it MAY be one second.
						; only gets to here 1 in every 256 times.
						; (this is our best optimised test)
						; it gets here when bres_mid ==0.

	tstf bres_hi		; test hi for zero too
	skpz				; z = both hi and mid are zero, is one second!
	goto int_exit		; nz, so not one second yet.

	;-------------------------------------------------
	; Only gets to here if we have reached one second.

	; now we can generate our one second event, like add
	; one second to our clock or whatever.
	; (in this example we toggle a led)

	; The other thing we need to do is add 1,000,000 counts
	; to our 24bit variable and start all over again.
	;-------------------------------------------------
						; Add the 1,000,000 counts first.
						; One second = 1,000,000 = 0F 42 40 (in hex)

						; As we know hi==0 and mid==0 this makes it very fast.
						; This is an optimised 24bit add, because we can
						; just load the top two bytes and only need to do
						; a real add on the bottom byte. This is much quicker
						; than a "proper" 24bit add.
						; get msb value and load in msb and mid

	init_v bres_hi, H'0F'
	init_p bres_mid, bmid

	movfw blo			; lsb value to add
	addwf bres_lo,f		; add it to the remainder already in lsb
	skpnc				; nc = no overflow, so mid is still ok

	incf bres_mid,f		; c, so lsb overflowed, so inc mid
						; this is optimised and relies on mid being known
						; and that mid won't overflow from one inc.

						; that's it! Our optimised 24bit add is done,
						; this is roughly twice as quick as a "proper"
						; 24bit add.
	;-------------------------------------------------
						; now we do the "event" that we do every one second.

						; Note! for this example we toggle a led, which
						; will give a flashing led which is on for a second
						; and off for a second.
						; Add your own code here for your one second event.

	btfss OPTIONS, 0	; thest if what mode is on	
	bsf OPTIONS, 6

	;-------------------------------------------------
	; now our one second event is all done, we can exit the
	; interrupt handler.
	;-------------------------------------------------
						; finally we restore w and status registers.
						; also clears TMRO int flag now we are finished.
int_exit

	BCF INTCON,T0IF		; reset the tmr0 interrupt flag

	movf status_temp,w  ; retrieve copy of STATUS register
	movwf STATUS        ; restore pre-isr STATUS register contents
	swapf w_temp,f
	swapf w_temp,w      ; restore pre-isr W register contents
	retfie				; return from interrupt
;------------------------------------------------------------------------------

;******************************************************************************
;  SETUP     (runs this only once at startup)
;******************************************************************************


setup

	;-------------------------------------------------
								; CMCON setup
	movlw  	0x07  				; Turn Comparator off and
	movwf 	CMCON 				; Enable pins for I/O	

	init_v2 bmid, H'42', blo, H'40'

								; OPTION setup
	movlw B'10001000'			;
		;   x-------			; 7, 0=enable, 1=disable, portb pullups
		;   -x------			; 6, 1=/, int edge select bit
		;   --x-----			; 5, timer0 source, 0=internal clock, 1=ext pin.
		;   ---x----			; 4, timer0 ext edge, 1=\
		;   ----x---			; 3, prescaler assign, 1=wdt, 0=timer0
		;   -----x--			; 2,1,0, timer0 prescaler rate select
		;   ------x-			;   000=2, 001=4, 010=8, 011=16, etc.
		;   -------x			; 
								; Note! We set the prescaler to the wdt, so timer0
								; has NO prescaler and will overflow every 256 
								; instructions and make an interrupt.

	clrf PORTA
	clrf PORTB

	bsf STATUS, RP0				; Select Bank 1	
	movwf OPTION_REG			; Load data into OPTION_REG

	;-------------------------------------------------
								; PORTS SETUP

	clrf PORTB					; init RB0-7
								; on PORT B all outputs are ground so 0, set W in binary
								; transfer from W to TRISB which is in fact PORTB (RB7-RB0 outputs)	
	movlw B'00100000'			; set W with wanted configuration for port PORTA (on 5 bytes)
	movwf PORTA					; transfer from W to TRISA/PORTA RA4-RA0 outputs

	bcf	STATUS, RP0				; bank 0
								; SET SDA AND SCL to high
	movlw H'FF'					; set SDA of DS1631 high
	movwf PORTA					; 

	;-------------------------------------------------
								; INTCON setup
								;
								; for this code example, we enable the timer0
								; overflow interrupt.
								;
								; enable interrupts last
								; interrupt setup
	movlw B'10100000'			; GIE=on TOIE=on (timer0 overflow int)
	movwf INTCON				; set up.

	;-------------------------------------------------
	; Note! Now the hardware is set up we need to load the
	; first count for one second into our 24bit bres variable.
	;-------------------------------------------------
								; Note! This example uses 4 MHz clock, which is
								; 1,000,000 counts per second.
								;
								; We require a 1 second period, so we must load
								; 1,000,000 counts each time.
								; 1,000,000 = 0F 42 40 (in hex)
								;
								; We also need to add 256 counts for the first time,
								; so we just add 1 to the mid byte.
								; Check mid overflow if needed.
	
								; here we load the 24bit variable.
	init_v bres_hi, H'0F'		; get msb value and put in hi 
								; get mid value (note we added 1 to it)
	init_p bres_mid, bmid
	incf bres_mid, 1	

	init_p bres_lo, blo		

								; now setup is complete, we can start execution.
	;-------------------------------------------------
								; VARIABLES SETUP
								;


	;-------------------------------------------------
								; 7654 3210
	init_v OPTIONS, H'80'		; 1000 0000		; [0] when user press the set button, only if what mode is active
												; [1] is when user pres set button, 
												; [2] not used
												; [3] means the programm has to set configuration for DS1631
												; [4] is if the temperature is negative
												; [5]=used by divide (divide is Done)
												; [6]=second event reached
												; [7]=time (hh:mm) is displayed.. 1 for true 0 for false 
	init_v2 seconds, D'0', sec_on_off, H'0'		; set seconds to 0 and sec_on_off to off
	;-------------------------------------------- set the clock	
	init_v2 h, D'12', m, D'00'					; set hour and minutes to 12:00
	;-------------------------------------------- 
	init_v t, B'11110101'						; 
	init_v2 tt, D'8', td, D'2'
	init_v2 te, D'3', date_type, D'1'
	;-------------------------------------------- set the date to 31 FE
	init_v2 dd, D'08', dm, D'08'				; 1 JA, 2 FE, 3 MA, 4 AP, 5 MA, 6 JU, 7 JL, 8 AU, 9 SE, 10 OC, 11 NO, 12 DE
	init_v2 year, D'10', hold, H'30'	
	call days_per_month_call					; init the days for the current month
	;-------------------------------------------- set the counters
	init_v2 WRT__CHG, H'5', WRT__WHAT, H'1'		; at every 6 seconds change displayed data (clock/date/temp)
												; if WRT__WHAT = 1, clock is displayd, if 1 date, if 2 temp				
	call test_WW								; initialize the segments with the displayed data
	;-------------------------------------------------
	init_v DS1631_Config, H'01'
	init_v2 tl, D'25',th, D'30'					; this is the minim/maxim thermostat
	call DS1631_Write_Configurations
	call DS1631_start_conversion_call

	goto main

	;-------------------------------------------------
	; Before this call (put_number_in_W) the W is initialized with 
	; another number from 0 to 2B (decimal: 0 to 44) see last column below
	; The subroutine returns in W another number: retlw 0x20 for example, puts in W 0x20 if W equals 0
	; 0x20 value is RB70=0010 0000 and will turn on leds to display 0 and the 7 segments
	;-------------------------------------------------

put_number_in_W 
	ADDWF 02h, 1
					;			RB07=BAGF :DCE	RB7=B RB6=A RB5=G RB4=F RB3=: RB2=D RB1=C RB0=E
	;numbers from 0 to 9
	retlw 0x28		;zero		RB70=0010 1000	0
	retlw 0x7D		;one		RB70=0111 1101	1
	retlw 0x1A  	;two		RB70=0001 1010	2
	retlw 0x19		;three		RB70=0001 1001	3
	retlw 0x4D		;four		RB70=0100 1101	4
	retlw 0x89		;five		RB70=1000 1001	5
	retlw 0x88		;six		RB70=1000 1000	6
	retlw 0x3D		;seven		RB70=0011 1101	7
	retlw 0x08		;eight		RB70=0000 1000	8
	retlw 0x09		;nine		RB70=0000 1001	9
	;letters from A to F (hexadecimal)
	retlw 0x0C		;A 			RB70=0000 1100	10 A
	retlw 0xC8		;b 			RB70=1100 1000	11 B
	retlw 0xAA		;C 				 1010 1010	12 C
	retlw 0x58		;d 			RB70=0010 1000	13 D
	retlw 0x8A		;E 			RB70=1000 1010	14 E
	retlw 0x8E		;F 			RB70=1000 1110	15 F
	;month1, these are letters
	;W returns month for month1 in range [16-27] so I'll have to add 16 to the month
	;when returning this value to the segment being displayed
	retlw 0x78		;J from JA	RB70=0011 1001	16 10
	retlw 0x8E		;F from FE	RB70=1000 1110	17 11
	retlw 0x2C		;|| from MA	RB70=0010 1100	18 12
	retlw 0x0C		;A from AP	RB70=0000 1100	19 13
	retlw 0x2C		;|| from MY	RB70=0010 1100	20 14
	retlw 0x78		;J from JU	RB70=0011 1000	21 15
	retlw 0x78		;J from JL	RB70=0011 1000	22 16
	retlw 0x0C		;A from AU	RB70=0000 1100	23 17
	retlw 0x89		;S from SE	RB70=1000 1001	24 18 
	retlw 0xD8		;o from oc	RB70=1101 1000	25 19 
	retlw 0xDC		;n from no	RB70=0111 1101	26 1A
	retlw 0x58		;d from dE	RB70=0010 1000	27 1B
	;month2
	;W returns month for month2 in ragne [28-39] so I'll have to add 28 to the month
	;when returning this value to the segment being displayed
	retlw 0x0C		;A from JA	RB70=0000 1100	28 1C
	retlw 0x8A		;E from FE	RB70=1000 1010	29 1D
	retlw 0x0C		;A from MA	RB70=0001 1001	30 1E
	retlw 0x0E		;P from AP	RB70=0000 1110	31 1F
	retlw 0x49		;Y from MY	RB70=1000 1001	32 20
	retlw 0x68		;U from JU	RB70=0100 1000	33 21
	retlw 0xEA		;L from JL	RB70=1110 1010	34 22
	retlw 0x68		;U from AU	RB70=0100 1000	35 23
	retlw 0x8A		;E from SE	RB70=1000 1010	36 24
	retlw 0xDA		;c from oc	RB70=1101 1010	37 25
	retlw 0xD8		;o from no	RB70=0111 1101	38 26
	retlw 0x8A		;E from dE	RB70=1000 1010	39 27
	;degress mark
	retlw 0x0F		;DEGREES	RB70=0000 1111	40 28
	;minus mark
	retlw 0xDF		;MINUS		RB70=1111 1011 	41 29
	retlw 0xCC		;h letter	RB70=1100 1100	42 2A
	retlw 0xDC		;n letter	RB70=1101 1100	43 2B
	retlw 0x58		;d letter	RB70=0101 1000	44 2C
	retlw 0x49		;y letter	RB70=0100 1001	45 2D
	retlw 0xCA		;t letter	RB70=1100 1010	46 2E
	retlw 0xEE		;l letter	RB70=1110 1110  47 2F

init_h
	init_v h, D'0'
	return

init_h_23
	init_v h, D'23'
	return

init_m
	init_v m, D'0'
	return

init_m_59
	init_v m, D'59'
	return

init_dd
	init_v dd, D'1'
	return

init_dm
	init_v dm, D'1'
	return

init_dm_12
	init_v dm, D'12'
	return

init_year
	init_v year, D'0'
	return

init_dy_99
	init_v year, D'99'
	return

init_dt
	init_v date_type, D'01'
	return

init_dt_02
	init_v date_type, D'02'
	return

init_tt
	init_v tt, D'2'
	return

init_tt_99
	init_v tt, D'99'
	return

init_td
	init_v td, D'0'
	return

init_td_99
	init_v td, D'99'
	return

init_te
	init_v te, D'0'
	return

init_te_99
	init_v te, D'99'
	return

init_tl
	init_v tl, D'0'
	return

init_tl_w_th
	init_p tl, th
	return

init_th
	init_p th, tl
	return

init_th_60
	init_v th, D'60'
	return

init_dd_month_days
	init_p dd, days_per_month
	decf dd, 1
	return

DS1631_start_conversion_call
	goto DS1631_start_conversion
back_from_start_conversion
	return

DS1631_read_call
	goto DS1631_get_temperature
back_from_get_temperature
	return

	;-------------------------------------------------
	; now our one SECOND EVENT
	; this is executed if no set up mode and when a second is reached
	;-------------------------------------------------

second_event
	bcf OPTIONS, 6
	incf seconds, 1				; increment second
	_if seconds, D'60', seconds_60

	decfsz WRT__CHG, 1			; decrement write change to test if eq 0
	goto WRT__CHG_not_0

search_display_next				; If WRT__CHG is 0 then search for another display thing
		incf WRT__WHAT, 1		; WRT__WHAT to + 1
		_if WRT__WHAT, H'4', SET_WRT__WHAT_to_1			
		decf WRT__WHAT, 0		; make W = WRT__WHAT - 1
		addlw tt				; make W = W + adress of tt
		movwf FSR
		movf INDF, 0			; put in W value of the address (tt, td or te)
		movwf WRT__CHG
		sublw H'00'				; decrement W to test if eq 0
		btfsc STATUS, Z			;-- this array can hold
			goto search_display_next
	_if WRT__WHAT, H'1', DS1631_start_conversion_call	
	_if WRT__WHAT, H'3', DS1631_read_call
	call test_WW				; initialize the segments to display next data/info
WRT__CHG_not_0
	call make_second_on_off
	goto back_from_int_second_event

	;-------------------------------------------------
	; Test if it is necessary to make the
	; seconds on or off
	; For this I'll take in consideration if on the display
	; it is shown the time this means WRT__WHAT is 1
	;-------------------------------------------------

make_second_on_off
	decfsz WRT__WHAT, 0			; if time is shown, only then make seconds middle two pints to switch
	goto skip_s_on				; switch on off the two middle points
	call s_on_off				
	goto skip_s_off
skip_s_on						; skipp if 0, while not eq 0 WRT__WHAT does not change
	clrf sec_on_off
skip_s_off
	return

	;-------------------------------------------------
	; WHAT Button push event
	; this is executed when user press WHAT Button
	;-------------------------------------------------

__what
	movfw hold						; Test to see if hold reached 0 and only then user will be able to 
	sublw H'0'						; "click" again
	btfss STATUS, Z
	return

	btfsc OPTIONS, 0				; Test if what mode already active, if yes skip if bit 0 of what_on is set
	goto skip_init_WRT__WHAT
									; Init WRT__WHAT
		init_v WRT__WHAT, H'3'		; Init WRT__WHAT to H'3'			
		init_v seconds, D'0'
		bsf OPTIONS, 0				; Make what mode active

skip_init_WRT__WHAT
		incf WRT__WHAT, 1			; If the button was pressed then increment WRT__WHAT
		init_v hold, H'30'			; Comment this if you want to check/test __set mode else it will never enter in that call
		_if WRT__WHAT, H'12', SET_WRT__WHAT_to_1

		call test_WW				; calling SET_WRT__WHAT_to_1 will also set what_on to false
		call make_second_on_off	
	return

	;-------------------------------------------------
	; SET MODE EVENT
	; this is executed when user press the SET BUTTON
	;-------------------------------------------------

__set
	movfw hold						; Test to see if holf reached 0 and only then user will be able to 
	sublw H'0'						; "click" again, this test is done because if you press a button
	btfss STATUS, Z					; it's function can be executed 100 times before you release it
	return

	btfss OPTIONS, 0				; Test to see if what mode is active, if it is then skip return
	return							; You cannot change +/- without a set mode active

	btfsc PORTB, 3					; Test PORTB[3], if no click then exit
	goto no_click_on_B3				; Comment test bit for further debug else will never enter the code
	goto __set_p					; If user pressed PORTB[3] then solve +
no_click_on_B3						; If no click on PORTB[3] then test the other button PORTA[5]
	btfsc PORTA, 5					; If no click then exit
	return							; If PORTA[5] was not pressed either then return
	goto __set_m					; else solve -

__set_plus
	goto __set
__set_p
		bsf OPTIONS, 1				; Set to true bit 3 from decrement something
		init_v hold, H'30'		
		call test_WW
		bcf OPTIONS, 1	
	return	

__set_minus
	goto __set
__set_m
		bsf OPTIONS, 2				; Set to true bit 3 from decrement something
		init_v hold, H'30'		
		call test_WW
		bcf OPTIONS, 2	
	return

;******************************************************************************
;  MAIN     (main program loop)
;******************************************************************************
;
;------------------
main
;------------------	
	btfsc OPTIONS, 6
	goto second_event
back_from_int_second_event

	init_v PORTA, H'1E'		; Set all four segments off: 00111111
	clrf PORTB				; Clear PORTB and make PORTB 0 & 3 inputs
	port_in_out_config BUTTONS_IN	
	time_waste H'01'

	btfss PORTB, 6			;test if what button was pressed, call __what if yes
	call __what
	call __set_plus			;test if set button was pressed, call __set if yes
	call __set_minus		;test if set button was pressed, call __set if yes

	port_in_out_config BUTTONS_OUT	
		
	init_v TL, H'4'			;H'10' means 16 H'4' means 4 TIME_LOOP
	call TIME_LOOP

	;-------------------------------------------------
	goto main				; keep running the main code.

blink_only_on_2_dig
	clrf sec_no
	movfw TL
	sublw H'2'
	btfsc STATUS, Z	
	goto back_from_blink

	btfsc sec_on_off, 3
	bsf sec_no, 0
	goto back_from_blink
;------------------------------------------------------------------------------

	;-------------------------------------------------
	; WRITE PORTA and PORTB
	; RETURN SEGMENT TO TURN ON
	; each 8 number is turned on at a time, but very fast
	; you have the impresion they are all turned on, this returns on Porta RA0/RA1/RA2/RA3
	; which will activate at a time
	; one segment display
	;-------------------------------------------------
write
	goto blink_only_on_2_dig
back_from_blink

	init_v PORTA, H'1E'		; Turn off all segments before begining to write
							; H'1F' = 0001 1110 RA4-1=1 make the transistors off, RA0=1 is the control pin for DS1820
	movlw H'20'				; Set 21 22 23 or 24 in W, seg1..4 addresses
	addwf TL, 0
	movwf FSR				; Return address of seg 1,2,3 or 4
	movf INDF, 0			; Put the number/value of seg 1,2,3 or 4 into W
	call put_number_in_W	; Return the coresponding configuration of PORTB for the specific number/value of W
							; Call write segment with
							; 0,1,2 or 3 from W and seg 1,2,3 or 4 value
							; For the two pints to light up, bit RB3 should be 1
	btfss sec_no, 0
	xorwf sec_on_off, 0		; For this bit to be 1, I will have to change from 0 to 1 or from 1 to 0	
	movwf PORTB

	goto back_to_TL				; from INDF, in fact I write on segment123or4 the coresponding number

TIME_LOOP
	goto write
back_to_TL
	 call set_next_segment

	init_v SERVICE, H'04'
LOOP_S
		movlw H'FF'
		call wasteit
		decfsz SERVICE, 1
	goto LOOP_S

	movfw hold
	sublw H'0'
	btfsc STATUS, Z
	goto over_dec			; Decrement hold, if it reaches 0 it will stay on 0
	decf hold, 1			; until what__ or set__ will init it to a value

over_dec
	decfsz TL, 1
	goto TIME_LOOP	
	return

	;-------------------------------------------------
	; RETURN SEGMENT TO TURN ON
	; each 8 number is turned on at a time, but very fast
	; you have the impresion they are all turned on but they are not
	;										RA  7654 3210	-> RA0 is connected to DS1820 control pin, always 1 (here)
	; RA4[1D]	->  command to display seg1		0001 1101	1D, the last 1 is RA0
	; RA3[1B]	->  command to display seg2		0001 1011	1B
	; RA2[17]	->  command to display seg3		0001 0111	17
	; RA1[0F]	->  command to display seg4		0000 1111	0F
	; RA4-1 are the ones connected to 2N3906 transistors that provide current to seg1 seg2 seg3 & seg4
	;-------------------------------------------------

set_next_segment
	movfw TL
	call set_active_segment 
	movwf PORTA	
	return

set_active_segment 
	btfsc TL, 2			; If bit 2=1 then TL equals 4 (0100)
		retlw H'0F'
	btfsc TL, 1			; Test bit 1 if 0 then skip
		goto test_3
	retlw H'1D'		; If bit 1=0 and bit 2=0 (tested above) then TL equals 1
test_3
	btfsc TL, 0
	retlw H'17'		; TL = 3
	retlw H'1B'		; TL = 2

	;-------------------------------------------------
	; WRITE WHAT?
	; As there are many things that can be written on the display 
	; WRT__WHAT will be used to hold an info about what will be displayed
	; There are 11 things that the display will show
	;-------------------------------------------------

test_WW
	decfsz WRT__WHAT, 1		; If WRT__WHAT is 10 decfsz will be 9 and 
	goto _2					; the next instruction goto_2 will not be skipped
	goto WW_is__1			; If decfsz is 0 the goto _x is skipped and will go to WWW_is_[x-1]
_2	decfsz WRT__WHAT, 1		; 
	goto _3
	goto WW_is__2			
_3	decfsz WRT__WHAT, 1
	goto _4
	goto WW_is__3
_4	decfsz WRT__WHAT, 1
	goto _5
	goto WW_is__4
_5	decfsz WRT__WHAT, 1
	goto _6
	goto WW_is__5
_6	decfsz WRT__WHAT, 1
	goto _7
	goto WW_is__6
_7	decfsz WRT__WHAT, 1
	goto _8
	goto WW_is__7
_8	decfsz WRT__WHAT, 1
	goto _9
	goto WW_is__8
_9	decfsz WRT__WHAT, 1
	goto _A
	goto WW_is__9
_A	decfsz WRT__WHAT, 1
	goto _B
	goto WW_is__A
_B	decfsz WRT__WHAT, 1
	goto _C
	goto WW_is__B
_C	decfsz WRT__WHAT, 1
	goto _D
	goto WW_is__C
_D	decfsz WRT__WHAT, 1
	goto _E
	goto WW_is__D
_E	decfsz WRT__WHAT, 1
	goto _F
	goto WW_is__E
_F	decfsz WRT__WHAT, 1
	goto _10
	goto WW_is__F
_10	decfsz WRT__WHAT, 1
	goto _11
	goto WW_is__10
_11	decfsz WRT__WHAT, 1
	return
	goto WW_is__11
	return

init_2segs macro number, s12, s34, dec_hex, get_WW
	init_v WRT__WHAT, get_WW
	divide number, dec_hex
	init_p s12, divide_Result
	init_p s34, divide_Rest
	endm

	;-------------------------------------------------
	;WRITE__WHAT is 1 : display the time hh:mm
	;-------------------------------------------------

WW_is__1
	init_2segs h, seg1, seg2, D'10', H'1'
	init_2segs m, seg3, seg4, D'10', H'1'
	return

	;-------------------------------------------------
	;WRITE__WHAT is 2 : display the date dd:mm
	;-------------------------------------------------

WW_is__2	
	init_2segs dd, seg1, seg2, D'10', H'2'
	init_2segs dm, seg3, seg4, D'10', H'2'
	decfsz date_type, 0
	return

	add dm, D'15', seg3
	add dm, D'27', seg4
	return	

	;-------------------------------------------------
	;WRITE__WHAT is 3 : display the temperature
	;Ex showing temperature: 
	;	 5gC where seg1=0 seg2=5 seg3=g C it is shown allways, will be seg4
	;	23gC where seg1=2 seg2=3 seg3=g and C
	;	-23C where seg1=- seg2=2 seg3=3 and C
	; 	-8gC where seg1=- seg2=8 seg3=g and C
	;-------------------------------------------------

divide_temp
	divide t, D'10'
	return

init_negative_temp_more_10
	bcf OPTIONS, 4						; Clear flag for temperature > -10 (EX: -10, -11 ..)
	init_v seg3, H'28'					; H'28' is DEGREES mark
	init_p seg2, divide_Rest			; if temperature > -10 (EX: -9 or -8 ..-1) the result 
	return								; is 0 so I will not display but rest of dividing 
										; at 10 is the temperature number
init_negative_temp_less_10
	init_p seg2, divide_Result			; if temperature <= -10 (EX: -10 or -11 ..) the result will be displayd on seg2
	init_p seg3, divide_Rest			; and the rest on seg3.. noo more place for degrees mark, seg4 will
	return								; display the Celsius mark

WW_is__3	
	init_v seg4, H'0C'					; seg4="C" Celsius mark, this is always shown
skip_init_I2C_to_3
	btfss t, 7							; Test Bit SIGN, skip if set, if it is 1 then the temperature is negative
	goto temperature_is_positive		; If it is 0 then the temperature is positive
	goto temperature_is_negative		
temperature_is_negative	
		bsf OPTIONS, 4					; Set negative temperature flag -11 -12 -13
		comf t, 1						; -10=11110101 so this must be complemented to be actually 10
		init_v seg1, H'29'				; but kipp seg1 equal to minus sign; H'29' is minus mark seg1="-"
		call divide_temp
		_if divide_Result, H'0', init_negative_temp_more_10
		btfsc OPTIONS, 4
		call init_negative_temp_less_10
		init_v WRT__WHAT, H'3'
		goto temp_done
temperature_is_positive	
	init_2segs t, seg1, seg2, D'10', H'3'
	init_v seg3, H'28'					; H'28' is DEGREES mark
temp_done	
	return

	;-------------------------------------------------
	;WRITE__WHAT is 4 : display setting hours
	;EX: ho:23 seg1seg2:seg3:seg4 seg1=h seg2=o seg3=2 seg4=3
	;-------------------------------------------------

WW_is__4								; Set hours
	btfss OPTIONS, 1					; If the set__+ button was pressed: btfss bit test, skip if set
	goto test_plus4						; If no button was pressed then only display the value for hours
		incf h, 1						; If the button was pressed then increment hours
		_if h, D'24', init_h			; If hours are 24 then hours=0
test_plus4
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__4
		decf h, 1						; 
		_if h, H'FF', init_h_23			; 
only_display__4
	init_v2 seg1, H'2A', seg2, H'19'	; H'2A' means h H'19' means o (see put_number_in_W) 
	init_2segs h, seg3, seg4, D'10', H'4'		; Init seg3 and seg4 with the hour
	return


	;-------------------------------------------------
	;WRITE__WHAT is 5 : display setting minutes
	;EX: nn:58 seg1seg2:seg3seg4 seg1=n seg2=n seg3=5 seg4=8
	;-------------------------------------------------

WW_is__5								; Set minutes
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plus5						; If no button was pressed then only display the value for minutes
		incf m, 1						; If the button was pressed then increment minutes
		_if m, D'60', init_m			; If minutes are 60 then minutes=0
test_plus5
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__5
		decf m, 1						; 
		_if m, H'FF', init_m_59			; 
only_display__5	
	init_v2 seg1, H'2B', seg2, H'2B'	; H'2B' means (see put_number_in_W) returning in PORTB n letter
	init_2segs m, seg3, seg4, D'10', H'5'		; retlw 0xD4		;n letter	RB70=1101 0100	38 26 (from put_number_in_W call)
	return

	;-------------------------------------------------
	;Init the day if exceeds days/month
	;If day is days_per_month+1 then make the day back to 1
	;EX: April has 30 days so if the day exceeds 30 it will be initialized to days_per_month
	;-------------------------------------------------

test_day_per_month
	movf days_per_month, 0				; Test to see if day is equal to
	subwf dd, 0							; the number of days supported by the month
	btfsc STATUS, Z						; If it is then
	call init_dd						; make dd = 1 else this instruction will be skipped
	return
	
	;-------------------------------------------------
	;When setting the month or the year there is a problem:
	;If the day is 31 and we change the month from March to April then
	;the day will not be valid for this month (April only has 30 days)
	;so	the day needs a little adjustment like one/or more decrements
	;Same problem is with the year, but only for february, when the
	;year modulo 4 is 0 then february has 28 days instead 29
	;-------------------------------------------------

test_day_for_month_year									
	init_p divide_Rest, days_per_month	; I need a service variable, so I'll use the divide_Rest, 
	decf divide_Rest, 1
test_dd_again							; this has nothing to do with the divide operation
	movfw divide_Rest					; I will substract from days_per_month (aka divide_Rest) the day (dd) and test if the result is 0
	subwf dd, 0							; If the result is not 0 then I will decrement days_for_month (aka divide_Rest) and substract again
	btfsc STATUS, Z						; and so on
	return								; return if the day (dd) is in range 0 < dd < days_per_month
										; If for one of the substraction the ressult is 0 the the day is in range if not,
	decfsz divide_Rest, 1				; the day is above the days_per_month and will need to be adjusted: equal to days_per_month
	goto test_dd_again	
	goto end_no_range
end_no_range
	init_p divide_Rest, days_per_month
	decf divide_Rest, 1
	init_p dd, divide_Rest
	return
	
	;-------------------------------------------------
	;WRITE__WHAT is 6 : display setting day from date
	;EX: dd:31 seg1seg2:seg3seg4 seg1=d seg2=d seg3=3 seg4=1
	;-------------------------------------------------

WW_is__6								; Set day from date
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plus6						; If no button was pressed then only display the value for day
		incf dd, 1						; If the button was pressed then increment day
		call test_day_per_month			; If day is more than the days supported by the month then day=1
test_plus6
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__6
		decf dd, 1						; 
		_if dd, H'00', days_per_month_call
		_if dd, H'00', init_dd_month_days
only_display__6
	init_v2 seg1, H'2C', seg2, H'2C'	; H'2C' means (see put_number_in_W) returning in PORTB d letter (d from date)
	init_2segs dd, seg3, seg4, D'10', H'6'		; put dd on seg3 and seg4 (dd/10=Result and Rest) seg3=Result seg4=Rest
	return	

	;-------------------------------------------------
	;WRITE__WHAT is 7 : display setting month from date
	;EX: dn:12 seg1seg2:seg3seg4 seg1=d seg2=n seg3=1 seg4=2
	;-------------------------------------------------

WW_is__7								; Set month from date
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plus7				; If no button was pressed then only display the value for month
		incf dm, 1						; If the button was pressed then increment month
		_if dm, D'13', init_dm			; if month is 13 then month=1
		call days_per_month_call		; After changing the month the days_per_month needs to be recalculated
		call test_day_for_month_year	; Test to see if the day is still in range 0 < dd <= days_per_month after recalculation of days_per_month
test_plus7
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__7
		decf dm, 1						
		_if dm, H'00', init_dm_12	
		call days_per_month_call		; After changing the month the days_per_month needs to be recalculated
		call test_day_for_month_year	; Test to see if the day is still in range 0 < dd <= days_per_month after recalculation of days_per_month	
only_display__7
	init_v2 seg1, H'2C', seg2, H'2B'	; H'2C' means (see put_number_in_W) returning in PORTB d letter (d from date)
	init_2segs dm, seg3, seg4, D'10', H'7'		; H'2B' means (see put_number_in_W) returning in PORTB n letter (n from month - could not display m on a sigle digit display)
	decfsz date_type, 0
	return

	add dm, D'15', seg3					; dm is for example 12 (this means december) and this will write to the display seg3: d seg4: E
	add dm, D'27', seg4					; adding 15 to dm see put_number_in_W 
	init_v WRT__WHAT, H'7'				; adding 27 to dm 
	return

	;-------------------------------------------------
	;WRITE__WHAT is 8 : display setting year from date
	;EX: dY:09 seg1seg2:seg3seg4 seg1=d seg2=Y seg3=0 seg4=9
	;-------------------------------------------------

WW_is__8								; Set year from date
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plus8						; If no button was pressed then only display the value for the year (only the last two 09 from 2009)
		incf year, 1					; If the button was pressed then increment month
		_if year, D'100', init_year		; This clock goes only to 2099
		call days_per_month_call		; After changing the year the days_per_month needs to be recalculated
		call test_day_for_month_year	; Test to see if the day is still in range 0 < dd <= days_per_month after recalculation of days_per_month
test_plus8
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__8
		decf year, 1						
		_if year, H'FF', init_dy_99	
		call days_per_month_call		; After changing the month the days_per_month needs to be recalculated
		call test_day_for_month_year	; Test to see if the day is still in range 0 < dd <= days_per_month after recalculation of days_per_month	
only_display__8
	init_v2 seg1, H'2C', seg2, H'2D'	; H'2C' means (see put_number_in_W) returning in PORTB d letter (d from date)
	init_2segs year, seg3, seg4, D'10', H'8'	; H'2D' means (see put_number_in_W) returning in PORTB Y letter (Y from year)
	return		

	;-------------------------------------------------
	;WRITE__WHAT is 9 : display setting date_type
	;EX: dt:01 seg1seg2:seg3seg4 seg1=d seg2=t seg3=0 seg4=1
	;If date type equals 01 then JA FE 03 AP 05 JU JL AU SE oc no dE are the months displayed
	;If date type equals 02 then numbers are used to display the month 01 02 03 04  05 06 07 08 09 10 11 12
	;-------------------------------------------------

WW_is__9								; Set date type
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plus9						; If no button was pressed then only display the value for the date type
		incf date_type, 1				; If the button was pressed then increment date type (only from 01 to 02)
		_if date_type, D'03', init_dt	; 
test_plus9
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__9
		decf date_type, 1						; 
		_if date_type, H'00', init_dt_02			; 
only_display__9	
	init_v2 seg1, H'2C', seg2, H'2E'	; H'2C' means (see put_number_in_W) returning in PORTB d letter (d from date)
										; H'2E' means (see put_number_in_W) returning in PORTB t letter (t from type)
	init_2segs date_type, seg3, seg4, D'10', H'9'		
	return

	;-------------------------------------------------
	;WRITE__WHAT is A (10) : display setting tt (time time) 
	;How long (in seconds, from 0 to 99) will display the clock hh:mm
	;EX: tt:10 seg1seg2:seg3seg4 seg1=t seg2=t seg3=1 seg4=0
	;-------------------------------------------------

WW_is__A								; Set second for how long to show time hh:mm
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plusA						; If no button was pressed then only display the value 
		incf tt, 1						; If the button was pressed then increment 
		_if tt, D'100', init_tt			; If time seconds are 100 then time seconds=0
test_plusA
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__A
		decf tt, 1						; 
		_if tt, H'1', init_tt_99			; 
only_display__A	
	init_v2 seg1, H'2E', seg2, H'2E'	; H'2E' means (see put_number_in_W) returning in PORTB t letter (t time, t time)
										; like time for showing time.. lol
	init_2segs tt, seg3, seg4, D'10', H'A'		; 
	return

	;-------------------------------------------------
	;WRITE__WHAT is B (11) : display setting td (time date) 
	;How long (in seconds, from 0 to 99) will display the date dd:mm
	;EX: td:10 seg1seg2:seg3seg4 seg1=t seg2=d seg3=1 seg4=0
	;-------------------------------------------------

WW_is__B								; Set second for how long to show date dd:mm
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plusB						; If no button was pressed then only display the value 
		incf td, 1						; If the button was pressed then increment 
		_if td, D'100', init_td			; If time seconds are 100 then time seconds=0
test_plusB
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__B
		decf td, 1						; 
		_if td, H'FF', init_td_99		
only_display__B	
	init_v2 seg1, H'2E', seg2, H'2C'	; H'2E' means t letter, H'2C' is d letter
	init_2segs td, seg3, seg4, D'10', H'B'		;
	return

	;-------------------------------------------------
	;WRITE__WHAT is C (12) : display setting te (time for temperature) 
	;How long (in seconds, from 0 to 99) will display the temperature tegC
	;EX: te:10 seg1seg2:seg3seg4 seg1=t seg2=e seg3=1 seg4=0
	;-------------------------------------------------

WW_is__C								; Set second for how long to show date dd:mm
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plusC						; If no button was pressed then only display the value 
		incf te, 1						; If the button was pressed then increment 
		_if te, D'100', init_te			; If time seconds are 100 then time seconds=0
test_plusC
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__C
		decf te, 1						; 
		_if te, H'FF', init_te_99		
only_display__C	
	init_v2 seg1, H'2E', seg2, H'1D'	; H'2E' means t letter, H'1D' is E letter
	init_2segs te, seg3, seg4, D'10', H'C'		;
	return

	;-------------------------------------------------
	;WRITE__WHAT is D (13) : display setting thermostat low value from :-50 to 0 C
	;EX: tL:20 seg1seg2:seg3seg4 seg1=t seg2=L seg3=2 seg4=0
	;-------------------------------------------------

WW_is__D								; Set minim thermostat TL
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plusD
		incf tl, 1						; If the button was pressed then increment tl (thermostat low)
		movfw th						; Test to see if tl>th and if it is then make it 0 (-50)
		addlw H'01'
		subwf tl, 0
		btfsc STATUS, Z
		call init_tl					; maximum value for thermostat low is 50 but negative to a minimum of 0	
		bsf OPTIONS, 3					; Used to know iof user set tl th or DS1631_Config
test_plusD
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__D
		decf tl, 1						; If the button was pressed then decrement tl (thermostat low)
		_if tl, H'FF', init_tl_w_th		; If tl = -1 aka FF then make it eq to th
		bsf OPTIONS, 3					; Used to know iof user set tl th or DS1631_Config
only_display__D
	init_v2 seg1, H'2E', seg2, H'22'	; 2E = t, 22 = L
	init_2segs tl, seg3, seg4, D'10', H'D'	
	return

	;-------------------------------------------------
	;WRITE__WHAT is E (14) : display setting thermostat high value from :1 to 60 C
	;EX: th:40 seg1seg2:seg3seg4 seg1=H seg2=- seg3=4 seg4=0
	;-------------------------------------------------

WW_is__E								; Set maxim thermostat TH
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plusE						; If no button was pressed then only display the value for the high thermostat
										; (only from 1 to 60)
		incf th, 1						; If the button was pressed then increment th (thermostat high)
		_if th, H'3D', init_th			; H'3D' means 61, maximum value for thermostat high is 60 to a minimum of tl
		bsf OPTIONS, 3					; Used to know if user set tl th or DS1631_Config
test_plusE
	btfss OPTIONS, 2
	goto only_display__E
		bsf OPTIONS, 3					; Used to know if user set tl th or DS1631_Config
		decf th, 1						; If the button was pressed then increment tl (thermostat low)

		decf tl, 0						; Test to see if th<tl and if it is then make it 0
		subwf th, 0
		btfsc STATUS, Z
		call init_th_60					; maximum value for thermostat low is 50 but negative to a minimum of 0	
only_display__E
	init_v2 seg1, H'2E', seg2, H'2A'	;H'2E' means t, H'2A' means h
	init_2segs th, seg3, seg4, D'10', H'E'
	return

	;-------------------------------------------------
	;WRITE__WHAT is F (15) : display setting thermostat active high or low 
	;User can select betwen 00 or 01 (00 means active low, 01 means active high)
	;EX: tA:01 seg1seg2:seg3seg4 seg1=t seg2=A seg3=0 seg4=1
	;-------------------------------------------------

WW_is__F								; Set thermostat active high or low
	
	btfss OPTIONS, 1					; If the set__ button was pressed: btfss bit test, skip if set
	goto test_plusF						; If no button was pressed then only display thermostat active/low high setting
										; 00 means active low and 1 means active high
	init_v DS1631_Config, H'03'
		bsf OPTIONS, 3					; Used to know if user set tl th or DS1631_Config
test_plusF
	btfss OPTIONS, 2					; If the set__- button was pressed: btfss bit test, skip if set
	goto only_display__F
	init_v DS1631_Config, H'01'
		bsf OPTIONS, 3					; Used to know if user set tl th or DS1631_Config
only_display__F
	init_v2 seg1, H'2E', seg2, H'0A'	; H'2E' returns H letter, H'0A' returns A letter
	init_2segs DS1631_Config, seg3, seg4, D'10', H'F'
	return

	;-------------------------------------------------
	;WRITE__WHAT 10 : display setting SH second high  
	;One second last 1.000.000 microseconds this is 0F4240 in hexadecimal
	;Second high is the middle group 42. User can adjust this middle group to
	;adjust/calibrate it's
	;duration over a second, up to a max of ~16960 microseconds +/-. If we take
	;in consideration that 00 is set also for low group.
	;If 0F0000 is set by the user (0F not setable) 
	;this means it's second will last 983040 microseconds
	;-------------------------------------------------

WW_is__10								; Set second high duration
	btfss OPTIONS, 1					; 
	goto test_plus10					
		incf bmid, 1					; If the button was pressed then increment 		
test_plus10
	btfss OPTIONS, 2					
	goto only_display__10
		decf bmid, 1						 
only_display__10	
	init_v2 seg1, H'18', seg2, H'2A'	; H'18' means S letter, H'25' is h letter
	init_2segs bmid, seg3, seg4, D'16', H'10'
	return

	;-------------------------------------------------
	;WRITE__WHAT is 11 : display setting SL second low 
	;One second last 1.000.000 microseconds this is 0F4240 in hexadecimal
	;This is the adjustements for the low nibble 40
	;This setting can make a 256 microseconds difference +/- from a million of us
	;-------------------------------------------------

WW_is__11								; Set second low duration
	btfss OPTIONS, 1					
	goto test_plus11					
		incf blo, 1								
test_plus11
	btfss OPTIONS, 2				
	goto only_display__11
		decf blo, 1								
only_display__11	
	init_v2 seg1, H'18', seg2, H'2F'	; H'18' means S letter, H'2B' is l letter
	init_2segs blo, seg3, seg4, D'16', H'11'
	return

	;-------------------------------------------------
	;Initialize WRT__WHAT to 1 when reached a maximum of 4 
	;in set mode off and B in set mode on
	;-------------------------------------------------

SET_WRT__WHAT_to_1
	bcf OPTIONS, 0
	init_v WRT__WHAT, H'1'
	movfw tt
	movwf WRT__CHG
	return

	;-------------------------------------------------
	;SET SET MODE OFF
	;-------------------------------------------------

	;Waste some time
wasteit
	addlw H'FF'			
	btfss STATUS, Z		
	goto wasteit		
	return

s_on_off
	movlw H'8'
	xorwf sec_on_off, 1			;on/off H'8'/H'0' from W to activate/dezactivate the two points
	return

divide_call
	init_v2 divide_Rest, H'0', divide_Result, H'0'
	movfw divide_Who	
	sublw H'0'					; Test if the number is 0
	btfsc STATUS, Z	
	return
	bcf OPTIONS, 5				; Kipp here divide_Done, when the divide operation is done then set this 
	init_v divide_Result, H'0'	; OPTIONS[5] flag true
d_again
	btfsc OPTIONS, 5
	goto d_done
	decfsz divide_Who, 1
	goto d_continue
	bsf OPTIONS, 5
d_continue
	incf divide_Rest, 1
	movf divide_Rest, 0
	subwf divide_At, 0
	btfss STATUS, Z				;skip next instruction if the result is 0
	goto d_again
d_next
	init_v divide_Rest, H'0'
	incf divide_Result, 1
	goto d_again
d_done
	return

	;-------------------------------------------------
	; SECONDS=60 EVENT & derivate subroutines 
	;-------------------------------------------------	

seconds_60						;if seconds=60 then secods=0
	init_v seconds, D'0'
	incf m, 1					;increment m in W, so W=m+1
	_if m, D'60', m_is_60		;test to see if m is 60
	_if h, D'24', h_is_24		;if h=24 increment days	
	goto init_day_per_month
back_from_init_day_per_month	;if dd=32 increment months	
	_if dm, D'13', dm_is_13		;if dm=13 increment years
	goto test_WW
	return

m_is_60
	init_v m, D'0'				;if m is 60 then m=0 
	incf h, 1					;and increment hours
	return

h_is_24							;if h1 is 4 then test is h2 is 2 (24 hour)
	init_v h, D'0'				;if m is 60 then m=0 
	incf dd, 1					;and increment hours	
	return

init_day_per_month
	call test_day_per_month
	btfsc STATUS, Z
	goto inc_dm
	goto back_from_init_day_per_month

inc_dm	
	incf dm, 1								;and increment months
	call days_per_month_call				;take days for current month
	goto back_from_init_day_per_month

dm_is_13
	init_v dm, D'1'							;if dd (day from date) is 32 then dd=1 
	incf year, 1							;and increment years
	call days_per_month_call
	return	

the_month_is_february
	init_v days_per_month, D'29'			;real value is 28
	divide year, D'4'
	_if divide_Rest, D'0', init_days_per_month_30
	return

init_days_per_month_31
	init_v days_per_month, D'31'			;real value is 30
	return

init_days_per_month_30
	init_v days_per_month, D'30'			;real value is 29
	return

days_per_month_call
	init_v days_per_month, D'32'			;real value is 31
	_if dm, H'4', init_days_per_month_31	;H'4' is April because dm starts from 1
	_if dm, H'6', init_days_per_month_31	;H'6' is June
	_if dm, H'9', init_days_per_month_31	;H'9' is Sep
	_if dm, H'B', init_days_per_month_31	;H'B' is Nov
	_if dm, H'2', the_month_is_february		;H'2' is Feb
	return

#include "DS1631.inc"

	END
